/*
 * Galactic Bloodshed, copyright (c) 1989 by Robert P. Chansky, 
 * smq@ucscb.ucsc.edu, mods by people in GB.c, enroll.dat.
 * Restrictions in GB.c.
 * vars.h --
 *  main bunch of variables 
 */

#include "files.h"
#include "tweakables.h"
#include <sys/types.h>
#include <sys/file.h>
#include <stdio.h>

#define LEVEL_UNIV 0
#define LEVEL_STAR 1
#define LEVEL_PLAN 2
#define LEVEL_SHIP 3

#define MAXPLAYERS 32
#define BITS_MAXPLAYERS 5	/* # bits req'd to represent (MAXPLAYERS-1) */

#define MAXSTRLEN 2047
#define HUGESTRLEN (2*MAXSTRLEN+1)
#define SMALLSTR 31

typedef hugestr[HUGESTRLEN];

typedef unsigned char percent;
typedef unsigned long ul;
typedef unsigned short us;
typedef char boolean;

long random();

typedef struct sector sectortype;
typedef struct planet planettype;
typedef struct star startype;

struct plinfo {				/* planetary stockpiles */
      unsigned short fuel;		/* fuel for powering things */
      unsigned short destruct;   	/* destructive potential */
      unsigned short resource;		/* resources in storage */
      unsigned short numsectsowned;

      unsigned comread : 7;		/* combat readiness (mobilization)*/
      unsigned explored : 1;		/* 1 bit: explored by each player */
      unsigned mob_set : 7;		/* mobilization target */
      unsigned autorep : 1;		/* player wants autoreport */
      unsigned tech_invest : 7;		/* amt of res. to spend on tech */
      unsigned spied_on : 1;		/* spied on */
      unsigned tox_thresh : 7;		/* min to build a waste can */
      unsigned dummy : 1;	/* dummy bit for development */

      unsigned int unused;     /* more dummy bits for development */
  };

struct sector {
    percent eff;		/* efficiency (0-100) */
    percent fert;		/* max popn is proportional to this */
    percent mobilization;	/* percent popn is mobilized for war */
    percent resource;	

    unsigned short popn;
    unsigned des : 3;		/* kind of land the place is */
    unsigned owner : BITS_MAXPLAYERS;	/* owner of place */
    unsigned is_wasted : 1;	/* 1 bit: is a wasted sector */
    unsigned VN : 1;		/* has a VN machine on it */
    unsigned dummy : 6; /* extra bits for development */
};


struct planet {
    float xpos,ypos;		/* x,y relative to orbit */
    short conditions[TOXIC+1];  /* atmospheric conditions for terraforming */
    unsigned short ships;       /* first ship in orbit (to be changed) */
    percent Maxx,Maxy;		/* size of map */
    int sectormappos;		/* file posn for sector map */

    struct plinfo info[MAXPLAYERS]; 	/* planetary stockpiles */

    unsigned int popn;
    unsigned int maxpopn;			/* maximum population */

    unsigned type : 3;		/* what type planet is */
    unsigned expltimer : 3;	/* timer for explorations */
    unsigned is_explored : 1;	/* is explored by owner */
    unsigned is_sheep : 1;	/* 1 bit: is emotion suppressed */
    unsigned slaved_to : BITS_MAXPLAYERS;	/* owner of place */
    unsigned dummy : 19;	/* dummy bits for development */
};

struct star {
    char name[NAMESIZE];	/* name of star */
    percent numplanets;		/* # of planets in star system */
    char pnames[MAXPLANETS][NAMESIZE]; /* names of planets */
    int planetpos[MAXPLANETS]; 		/* file posns of planets */
    unsigned int explored; 	/* who's been here 32 bits*/
    unsigned int inhabited;	/* who lives here now 32 bits*/
    percent stability;		/* how close to nova it is */
    unsigned short ships;			/* 1st ship in orbit */
    percent AP[MAXPLAYERS];	/* action pts alotted */
    unsigned nova_stage : 4;		/* stage of nova */
    unsigned temperature : 4;  /* factor which expresses how hot the star is */
    percent dummy;		/* dummy bits for development */
    float gravity;		/* attraction of star in "Standards". */
    float xpos,ypos;
};



	/* this data will all be read at once */
struct stardata {
    int numstars;		/* # of stars */
    unsigned short ships;		/* 1st ship in orbit */
    percent AP[MAXPLAYERS];	/* Action pts for each player */
    unsigned short VN_hitlist[MAXPLAYERS];	 
			/* # of ships destroyed by each player */
    percent VN_index1[MAXPLAYERS];
    percent VN_index2[MAXPLAYERS];	/* VN's record of destroyed ships
					   systems where they bought it */
};

EXTERN struct stardata Sdata;

struct directory {
    unsigned level : 2;	 /* what directory level */
    unsigned int gag : 1;/* whether or not to display login/logout messages */
    int snum;		/* what star system obj # (level=0) */
    int pnum;		/* number of planet */
    int shipno;		/* # of ship */
    char prompt[3*NAMESIZE+5];	/* just to be safe */
    float lastx[2], lasty[2], zoom[2];	/* last coords for zoom */
};

EXTERN struct directory Dir[MAXPLAYERS];

EXTERN sectortype Smap[(MAX_X+1)*(MAX_Y+1) + 1];

extern float powscale(),logscale(),float_rand(),gravity();
char *malloc();

EXTERN boolean Nuked[MAXPLAYERS];
EXTERN boolean StarsInhab[NUMSTARS];
EXTERN startype *Stars[NUMSTARS];
EXTERN unsigned short Sdatanumships[MAXPLAYERS];
EXTERN unsigned int Sdatapopns[MAXPLAYERS];
EXTERN unsigned short starnumships[NUMSTARS][MAXPLAYERS];
EXTERN unsigned int starpopns[NUMSTARS][MAXPLAYERS];

EXTERN int tot_resdep, prod_eff, prod_res[MAXPLAYERS];
EXTERN int prod_fuel[MAXPLAYERS], prod_destruct[MAXPLAYERS];
EXTERN int tot_captured, prod_mob, populations[MAXPLAYERS];
EXTERN float avg_mob[MAXPLAYERS];
EXTERN char sects_gained[MAXPLAYERS], sects_lost[MAXPLAYERS];
EXTERN boolean Claims;

EXTERN planettype *planets[NUMSTARS][MAXPLANETS];
EXTERN float Compat[MAXPLAYERS];
EXTERN int Num_races, Num_ships;

EXTERN int Two[32];	/* powers of 2 for bit operations */

/* bit routines stolen from UNIX <sys/param.h> */
#define	setbit(a,i)	((a) |= Two[(i)-1])
#define	clrbit(a,i)	((a) &= ~Two[(i)-1])
#define	isset(a,i)	((a) & Two[(i)-1])
#define	isclr(a,i)	(((a) & Two[(i)-1]) == 0)

